<?php
/**
 * @package Keyinvoice
 */

namespace KeyinvoiceWordpress\API;

use SoapClient;

class Keyinvoice 
{
    public $url;
    public $soap;
    public $sid;
    public $allow_final_consumer;
    private $behaviour_name = 'Simples';

    public function __construct( $url )
    {
        $this->allow_final_consumer     = get_option('api_final_consumer');
        $this->type                     = get_option('api_doctype'); 
        $this->draft                    = get_option('api_draft');
        $this->emaildocument            = get_option('api_send_email');
        $this->api_docref               = get_option('api_docs_ref');

        $this->url = $url;
        $this->soap = new SoapClient($this->url, array('cache_wsdl' => WSDL_CACHE_NONE) );
    }
    
    public function authenticate( $api_key )
    {
        $response = $this->soap->authenticate( $api_key );
        $this->sid = $response[1];
        return $response;
    }


    public function company( $sid )
    {
        return $this->soap->company( $sid );
    }

    public function countProducts()
    {
        return $this->soap->countProducts( $this->sid );
    }

    public function listProducts( $offset )
    {
        return $this->soap->listProducts(  $this->sid, $offset );
    }

    public function generateInvoice( $order_id )
    {
        $order = wc_get_order( $order_id );

        if( !$order ){ die("error, no order"); }

        $address = $order->get_address();
        $shipping_address = $order->get_address('shipping');

        $countries = new \WC_Countries();
        $all_countries = $countries->get_countries();

        $final_consumer = FALSE;
        $client_ok      = FALSE;
     
        $nif = $order->get_meta('_billing_keyinvoice_vat_field_code');

        if(trim($nif) != '')
        {            
            if( $this->verifyClient($nif) ) //NAO EXISTE
            {
                $this->updateClient($nif,$address);
                $clientok = TRUE;

            }else
            {
                if( $order->billing_country=='PT' || ($order->billing_country=='' && $countries->get_base_country()=='PT') )
                {
                    $result = $this->insertClient($nif,$address); 
                    if($result === TRUE)
                    {
                        $clientok = true;
                        $this->insertAltAddress($nif,$shipping_address);
                    }else
                    {
                        return '<div class="error notice"><p>Error, ao criar cliente! #132</p></div>';                                   
                    }  

                }else
                {
                    if($order->billing_country != '')
                    {
                        if(!$this->verifyCountry( $order->billing_country ))
                        {
                            $this->insertCountry( $order->billing_country, utf8_encode($all_countries[$order->billing_country]) );
                        }

                        $result = $this->insertForeignClient($nif,$address,$order->billing_country);
                        if( $result === TRUE )
                        {
                            $clientok = true;
                        }else
                        {
                            return '<div class="error notice"><p>Erro, ao inserir cliente estrangeiro.  #134</p></div>';   
                        }
                    }     
                }
                
            }	

        }else
        {
            $final_consumer = TRUE;
        } 

        if($clientok || ($final_consumer && $this->allow_final_consumer)) 
        {
            $api_prices_with_vat = 0;
            if($result = $this->verifyUserInsertionPricesWithVAT())
            {
                $api_prices_with_vat = $result;
            }    
            
            $product_price_with_vat = 0;
            if($result = $this->verifyPriceLevelVAT())
            {
                $product_price_with_vat = $result;
            }
        }

        if($final_consumer && $this->type == 13) //SE FOR FINAL CONSUMER E ENCOMENDA NÃO Dá
        {
            return '<div class="error notice"><p>'.__('Não é possível exportar o documento do tipo Encomenda para um consumidor final.').'</p></div>';
        }  

        $docref='';
        if(trim($this->api_docref) != '')
        {
            $docref = trim($this->api_docref);
        }
        $params =   [ 
                        'nif' => $nif,
                        'docType' => $this->type,
                        'obs' => utf8_encode("WOOCOMMERCE Encomenda Nº ".$order->id.' ('.$order->order_key.')'),
                        'opt_name' => utf8_encode($address['first_name'].' '.$address['last_name']),
                        'opt_nif' =>"",
                        'opt_address' =>utf8_encode($address['address_1'].' '.$address['address_2']), 
                        'opt_locality' => utf8_encode($address['city']), 
                        'opt_postalCode' =>utf8_encode($address['postcode']), 
                        'docRef' => $docref.$order->id.'_'.$order->order_key 
                    ];  

        $result = $this->insertDocumentHeader($params);

        if($result > 0)
        {
            $doc_temp_id = $result;
            $items = $order->get_items();

            foreach( $items as $item)
            {
                write_log('ITEM; '. $item);
                $tax_rate = 23;
                $product_price = 0;
                $discount = array();
                $final_discount = '';
                $product   = $order->get_product_from_item( $item );

                if( $product->is_taxable()) 
                {         
                    $tax_rates  = \WC_Tax::get_rates( $product->get_tax_class() );
                    $tax_rates = array_values(array_filter($tax_rates));
                    if(!empty($tax_rates))
                    {
                        if(isset($tax_rates[0]))
                            $tax_rate = $tax_rates[0]['rate'];
                    }                         
                }

                if($product_price_with_vat) $product_price = $item['subtotal_tax']+$item['subtotal'];
                else  $product_price = $item['subtotal']; 

                if( $product_exists = $this->productExists($product->get_sku()))
                { 
                    $prod = $this->getProduct($product->get_sku());
                    $prod_info = $this->getProduct_additionalInfo($product->get_sku());

                    if(!$prod_info)
                    {
                        return '<div class="error notice"><p>'.__('Não é possível exportar o documento.Erro ao carregar informação do produto.').'</p></div>';   
                    }  
                    
                    if($prod_info->BehaviourType != $this->behaviour_name)
                    {
                        return '<div class="error notice"><p>'.__('Não é possível exportar o documento. Erro ao carregar informação do produto.').'</p></div>';   
                    }  
                }else
                {
                    $params = [ 
                        'ref' => $product->get_sku(),
                        'designation' => utf8_encode($product->get_title()),
                        'shortName' => utf8_encode($product->get_title()),
                        'tax' => $tax_rate,
                        'obs' => '',
                        'isService' => '0',
                        'hasStocks' => '1',
                        'active' => '1',
                        'shortDesc' => utf8_encode(htmlspecialchars_decode($product->get_post_data()->post_excerpt)),
                        'longDesc' => utf8_encode(htmlspecialchars_decode($product->get_post_data()->post_content)),
                        'price' => $product_price,
                        'vendorRef' => '',
                        'ean' => $product->get_sku()
                    ];
                    if(!$this->insertProduct($params))
                    {  
                        return '<div class="error notice"><p>'.__('Não foi possível continuar a exportação da encomenda. Erro ao criar o produto REF: "'.$product->get_sku().'".').'</p></div>';   
                    } 
                }
                // if($product->get_image_id())
                // {
                //     if(get_post_mime_type($product->get_image_id()) == 'image/jpeg')
                //     {  
                //         $img_src = wp_get_attachment_image_src($product->get_image_id());        
                //         if(file_exists(wp_upload_dir()['path'].'/'.basename($img_src[0])) && is_file(wp_upload_dir()['path'].'/'.basename($img_src[0])))
                //         {    
                //             $this->insertProductImageByURL($product->get_sku(),$img_src[0]);
                //         }
                //     }
                // }

                $categories = wp_get_object_terms( $product->id, 'product_cat',array('orderby'=>'parent') );
                $family_to_connect = 0;
                $level = 0;

                if($categories)
                {
                    foreach($categories as $category)
                    {
                        $level_category = 0;
                        $idparentfamily = '';
                        
                        $fathers = $this->get_category_fathers($category->term_id);

                        if(!empty($fathers))
                        {
                            $fathers = array_reverse($fathers);
                            foreach($fathers as $father)
                            {
                                $f = get_term( $father, 'product_cat' );
                                $search = $this->familySearch('',$f->name);
                                if(!$search)
                                {
                                    if($family_insert = $this->insertFamily(utf8_encode($f->name),utf8_encode($f->name),$idparentfamily))
                                    {  
                                        $idparentfamily = $family_insert;
                                    }    
                                }else
                                {
                                    $idparentfamily = $search;
                                }
                                $level_category++;
                            }    
                        }
                        $family = $this->familySearch('',$category->name);
                        if(!$family)
                        {                            
                            $family_insert = $this->insertFamily(utf8_encode($category->name),utf8_encode($category->name),$idparentfamily); 
                            if($family_insert > 0)
                            {
                                if($level_category >= $level)
                                $family_to_connect  = $family_insert;
                            }else
                            {
                                return '<div class="error notice"><p>Erro, ao inserir categoria</p></div>';  
                            }
                        }else
                        {
                            if($level_category >= $level)
                                $family_to_connect  = $family;
                        }
                    }

                    if(intval($family_to_connect))
                        $this->setProductField($product->get_sku(),'familyid',$family_to_connect);   
                }

                if($product->is_on_sale())
                {
                    $discount[] = round(((($product->get_price_excluding_tax(1,$product->regular_price) - $product->get_price_excluding_tax())/$product->get_price_excluding_tax(1,$product->regular_price))*100),2);
                }  

                if($api_prices_with_vat) $product_price = $product->get_price_including_tax(1,$product->regular_price);
                else  $product_price = $product->get_price_excluding_tax(1,$product->regular_price);

                if($order->get_item_subtotal($item) != $order->get_item_total($item)) //COUPONS
                {
                    $discount[] = round(((($order->get_item_subtotal($item) - $order->get_item_total($item))/$order->get_item_subtotal($item))*100),2);
                }

                $final_discount = implode('+',$discount);
                $final_discount = str_replace(" ","",$final_discount);

                $prod_desc = utf8_encode($product->get_title());

                $params = [
                    'idDocTemp' => $doc_temp_id,
                    'docType' => $this->type,
                    'ref' => $product->get_sku(),
                    'qt' => $item['qty'],
                    'price' => $product_price,
                    'tax' => $tax_rate,
                    'prodDesc' => $prod_desc,
                    'discount' => $final_discount
                ]; 

                $insert_document_line = $this->insertDocumentLine($params);   
                if($insert_document_line !== TRUE)
                {
                    return '<div class="error notice"><p>Erro ao inserir linha do documento</p></div>';  
                }
            }

            
            $shipping_rate = 0;
            $shipping = $order->get_total_shipping();

            write_log('SHIPPING '.$shipping);
            if($shipping != 0)
            {
                $shipping_tax   = $order->get_shipping_tax();
                write_log('SHIPPING TAX '.$shipping_tax);
                if($shipping_tax != 0)
                {
                    $shipping_rate  = floatval(round((($shipping_tax/$shipping)*100)));
                }
                write_log('SHIPPING rat e'.$shipping_rate);
                if($api_prices_with_vat) $shipping += $shipping_tax;
                // NAO INSERE PREÇOS COM IVA
                $params = array(
                    'idDocTemp' => $doc_temp_id,
                    'docType' => $this->type,
                    'ref' => '',
                    'qt' => 1,
                    'price' => $shipping,
                    'tax' => $shipping_rate,
                    'prodDesc' => __('Portes de Envio'),
                    'discount' => ''
                ); 

                $insert_document_line = $this->insertDocumentLine($params);   
                if($insert_document_line !== TRUE)
                {
                    return '<div class="error notice"><p>Erro ao inserir linha, portes de envio</p></div>';  
                } 
                
            }

            if(!$this->draft)
            {  
                $doc_result = $this->closeDocument($doc_temp_id,$this->type);
                if( !$doc_result )
                {
                    return '<div class="error notice"><p>'.__('Não foi possível finalizar o documento da encomenda.').'</p></div>'; 
                }
                
                $document = $this->getDocument($doc_result,$this->type);
                if( !$document )
                {   
                    return '<div class="error notice"><p>'.__('Não foi possível associar o documento da encomenda.').'</p></div>';  
                }

                $document_key = $document->IdDoc.'_'.$document->DocType.'_'.$document->DocSeries; 

                if ( ! add_post_meta( $order->id, '_keyinvoice_document_id', $document_key , true ) ) { 
                    update_post_meta( $order->id, '_keyinvoice_document_id', $document_key );
                }

                if( $this->emaildocument && $address['email'] != "" )
                {
                    $body = utf8_encode('Caro(a) '.$address['first_name'].' '.$address['last_name'].', <br/><br/>
                                        Enviamos em anexo a factura relativa à sua encomenda nº '.$order->id.', datada de '.$order->date_created->date('Y-m-d H:i').'. <br/>');

                    $result = $this->sendDocumentPDF2Email( $doc_result, 
                                                                $this->type, 
                                                                utf8_encode($address['email']), 
                                                                utf8_encode('Envio do documento relativo à encomenda nº '.$order->id),
                                                                $body );
                    if( $result !== TRUE )
                    {
                        return '<div class="error notice"><p>Erro ao enviar email</p></div>';   
                    }
                }
            }


        }else
        {
            return '<div class="error notice"><p>Erro ao inserir cabeçalho do documento</p></div>';
        } 
        return '<div class="updated notice"><p>'.__('A encomenda foi exportada com sucesso.').'</p></div>';
    }

    public function verifyClient($nif)
    {
        $result = $this->soap->clientExists($this->sid,$nif);
        if($result[0] == 1)
        {
            return TRUE;
        }
        else
        {
            return FALSE; 
        }
    }
    
    public function updateClient($nif, $address)
    {
        $this->soap->updateClient(  
            $this->sid, 
            $nif, 
            utf8_encode($address['first_name'].' '.$address['last_name']), 
            utf8_encode($address['address_1'].' '.$address['address_2']), 
            utf8_encode($address['postcode']), 
            utf8_encode($address['city']), 
            $address['phone'], 
            '',
            $address['email'],
            'API->WOOCOMMERCE' 
        );
    }

    public function insertAltAddress($nif,$shipping_address)
    {
        if($shipping_address['country'] == 'PT')
        {
            $this->soap->insertAltAddress( 
                $this->sid, 
                $nif, 
                utf8_encode($shipping_address['address_1'].' '.$shipping_address['address_2']), 
                utf8_encode($shipping_address['postcode']), 
                utf8_encode($shipping_address['city'])
            );
        }
    }

    public function verifyCountry( $country_name )
    {
        $result = $this->soap->countryExists($this->sid,$country_name);
        if($result[0] == 1)
        {
            return TRUE;
        }
        else
        {
            return FALSE;
        }
    }

    public function insertClient($nif,$address)
    {
        $result = $this->soap->insertClient( 
            $this->sid, 
            $nif, 
            utf8_encode($address['first_name'].' '.$address['last_name']), 
            utf8_encode($address['address_1'].' '.$address['address_2']), 
            utf8_encode($address['postcode']), 
            utf8_encode($address['city']), 
                $address['phone'], 
            '',
            $address['email'],
            'API->WOOCOMMERCE'
        );
        if( $result[0] == 1 )
        {
            return TRUE;
        }
        return $result[0];
    }

    public function insertCountry($code,$name)
    {
        $this->soap->insertCountry( $this->sid, $code, $name );
    }

    public function insertForeignClient($nif,$address,$country_code)
    {
        $result = $this->soap->insertForeignClient( 
            $this->sid, 
            $nif, 
            $country_code,
            utf8_encode($address['first_name'].' '.$address['last_name']), 
            utf8_encode($address['address_1'].' '.$address['address_2']), 
            utf8_encode($address['postcode']), 
            utf8_encode($address['city']),  
            $address['phone'], 
            '',
            $address['email'],
            'API->WOOCOMMERCE' 
        );
        if($result[0] == 1)
        {
            return TRUE;
        } 
        return $result[0];
    }
    
    public function verifyUserInsertionPricesWithVAT()
    {
        $result = $this->soap->verifyUserInsertionPricesWithVAT($this->sid);
        if($result[0] == 1)
        {
            return $result[1];
        } 
        return FALSE; 
    }

    public function verifyPriceLevelVAT()
    {
        $result = $this->soap->verifyPriceLevelVAT($this->sid);
        if($result[0] == 1)
        {
            return $result[1];
        } 
        return FALSE;
    }

    public function insertDocumentHeader($params)
    {
        $result = $this->soap->insertDocumentHeader( 
            $this->sid, 
            $params['nif'],
            $params['docType'],
            $params['obs'],
            $params['opt_name'],
            $params['opt_nif'],
            $params['opt_address'],
            $params['opt_locality'],
            $params['opt_postalCode'],
            $params['docRef']
        );
        if($result[0] == 1)
        {
           return $result[1];
        } 
        return $result[0];
    }

    public function productExists($ref)
    {
        $result = $this->soap->productExists($this->sid,$ref);
        if($result[0] == 1)
        {
            return TRUE;
        }
        return FALSE;   
    }

    public function getProduct($ref)
    {
        $result = $this->soap->getProduct($this->sid,$ref);
        if($result->RC == 1)
        {
            return $result->DAT[0];
        } 
        return FALSE;
    }

    public function getProduct_additionalInfo($ref)
    {
        $result = $this->soap->getProduct_additionalInfo($this->sid,$ref);
        if($result->RC == 1)
        {
            return $result->DAT;
        } 
        return FALSE;
    }

    public function insertProduct($params)
    {
        $result = $this->soap->insertProduct(
            $this->sid,
            $params['ref'],
            $params['designation'],
            $params['shortName'],
            $params['tax'],
            $params['obs'],
            $params['isService'],
            $params['hasStocks'],
            $params['active'],
            $params['shortDesc'],
            $params['longDesc'],
            $params['price'],
            $params['vendorRef'],
            $params['ean']
        );
        if($result[0] == 1)
        {
            return TRUE;
        }
        return FALSE;    
    }

    public function get_category_fathers( $id,$chain= array())
    {
        $parent = get_term( $id, 'product_cat' );
        if ( $parent->parent && ( $parent->parent != $parent->term_id ) ) 
        {            
            $chain[] = $parent->parent; 
            $chain = $this->get_category_fathers( $parent->parent,$chain);
        }
        return $chain;     
    }

    public function familySearch($name,$value)
    {
        $result = $this->soap->familySearch($this->sid,$name,$value);
        if($result[0] == 1)
        {
            return $result[1];
        }
        return FALSE;
    }

    public function insertFamily($name,$ref,$idparentfamily)
    {
        $result = $this->soap->insertFamily($this->sid,$name,$ref,$idparentfamily);
        if($result[0] == 1)
        {
            return $result[1];
        } 
        return $result[0];
    }

    public function setProductField($ref,$fieldname,$value)
    {
        $this->soap->setProductField($this->sid,$ref,$fieldname,$value);    
    }

    public function closeDocument($doc_temp_id,$doc_type)
    {
       $result = $this->soap->closeDocument($this->sid,$doc_temp_id,$doc_type);
       if($result[0] == 1)
       {
          return $result[1];
       } 
       return FALSE;
    }
    
    public function getDocument($doc,$type)
    {
        $result = $this->soap->getDocument($this->sid,$doc,$type);
        if($result->RC == 1)
        {
            return $result->DAT[0];
        } 
        return FALSE;
    }

    public function sendDocumentPDF2Email($doc,$type,$email,$subject,$body)
    {
       $result = $this->soap->sendDocumentPDF2Email($this->sid,$doc,$type,$email,$subject,$body);
       if($result[0] == 1)
       {
          return TRUE;
       }
       return $result;
    }

    public function insertDocumentLine($params)
    {
        $result = $this->soap->insertDocumentLine( 
            $this->sid, 
            $params['idDocTemp'],
            $params['docType'],
            $params['ref'],
            $params['qt'],
            $params['price'],
            $params['tax'],
            $params['prodDesc'],
            $params['discount']
        );
        
        if($result[0] == 1)
        {
            return TRUE;
        }
        return $result[0];
    }

    public function getPDFLink($document)
    {
       $keyinvoice_document_ids = explode('_',$document);
       $result = $this->soap->getDocumentPDFLink_bySeries(
            $this->sid,
            $keyinvoice_document_ids[0],
            $keyinvoice_document_ids[1],
            $keyinvoice_document_ids[2]
        );
        if($result[0] == 1)
        {
            return $result[1];
        } 
        return FALSE; 
    }
      
}
