<?php
/**
 * @package Keyinvoice
 */

namespace KeyinvoiceWordpress\API;

class Wp 
{
    public $admin_pages = array();
    public $admin_subpages = array();
    public $settings = array();
    public $sections = array();
    public $fields = array();

    public function register()
    {
        if( !empty( $this->admin_pages ) )
        {
            add_action( 'admin_menu', [ $this , 'addAdminMenu' ] );
        }

        if( !empty( $this->settings ) )
        {
            add_action( 'admin_init', [ $this, 'registerCustomFields' ] );
        }
    }

    public function addPages( array $pages )
    {
        $this->admin_pages = $pages;
        return $this;
    }

    public function withSubPage( string $title = null )
    {
        if( empty( $this->admin_pages ) )
        {
            return $this;
        }

        $admin_page = $this->admin_pages[0];

        $subpage = [
            [
                'parent_slug' => $admin_page['menu_slug'], 
				'page_title' => $admin_page['page_title'], 
				'menu_title' => ($title) ? $title : $admin_page['menu_title'], 
				'capability' => $admin_page['capability'], 
				'menu_slug' => $admin_page['menu_slug'], 
				'callback' => $admin_page['callback']
            ]
        ];
        $this->admin_subpages = $subpage;
        return $this;
    }

    public function addSubPages(  array $pages )
    {
        $this->admin_subpages = array_merge( $this->admin_subpages, $pages );
        return $this;
    }

    public function addAdminMenu()
    {
        foreach ($this->admin_pages as $page) 
        { 
            add_menu_page( $page['page_title'], $page['menu_title'], $page['capability'], $page['menu_slug'], $page['callback'], $page['icon_url'], $page['position'] );
        }

        foreach ($this->admin_subpages as $page ) 
        { 
            add_submenu_page( $page['parent_slug'], $page['page_title'], $page['menu_title'], $page['capability'], $page['menu_slug'], $page['callback'] );
        }
    }
    
    public function setSettings( array $settings )
	{
		$this->settings = $settings;
		return $this;
	}

	public function setSections( array $sections )
	{
		$this->sections = $sections;
		return $this;
	}

	public function setFields( array $fields )
	{
		$this->fields = $fields;
		return $this;
	}

    public function registerCustomFields()
    {
        foreach( $this->settings as $setting )
        {
            register_setting( $setting["option_group"], $setting["option_name"], ( isset( $setting["callback"] ) ? $setting["callback"] : '' ) );
        }

        foreach( $this->sections as $section )
        {
            add_settings_section( $section["id"], $section["title"], ( isset( $section["callback"] ) ? $section["callback"] : '' ), $section["page"] );

        }

        foreach( $this->fields as $field )
        {
            add_settings_field( $field["id"], $field["title"], ( isset( $field["callback"] ) ? $field["callback"] : '' ), $field["page"], $field["section"], ( isset( $field["args"] ) ? $field["args"] : '' ) );
        }
    }

    public function setProduct( $args, $sku = false )
    {

        if( isset($sku) && !empty($sku) && $sku )
        {
            $product_id = wc_get_product_id_by_sku( $sku );
        }

        if( $product_id )
        {
            $product = wc_get_product( $product_id );

        }else
        {
            
            $product = new \WC_Product_Simple();
        }
        
        $product->set_name( utf8_decode($args['name']) ); 
        if( isset( $args['slug'] ) )
            $product->set_name( $args['slug'] );
  
        $product->set_description( utf8_decode($args['description']) );
        $product->set_short_description( utf8_decode($args['short_description']) );

        $product->set_status( isset($args['status']) ? $args['status'] : 'publish' );
        $product->set_catalog_visibility( isset($args['visibility']) ? $args['visibility'] : 'visible' );      
        $product->set_featured(  isset($args['featured']) ? $args['featured'] : false );
        $product->set_virtual( isset($args['virtual']) ? $args['virtual'] : false );

        $product->set_regular_price( $args['regular_price'] );
        $product->set_sale_price( isset( $args['sale_price'] ) ? $args['sale_price'] : '' );
        $product->set_price( isset( $args['sale_price'] ) ? $args['sale_price'] :  $args['regular_price'] );
        if( isset( $args['sale_price'] ) ){
            $product->set_date_on_sale_from( isset( $args['sale_from'] ) ? $args['sale_from'] : '' );
            $product->set_date_on_sale_to( isset( $args['sale_to'] ) ? $args['sale_to'] : '' );
        }
        
        $product->set_downloadable(  isset($args['downloadable']) ? $args['downloadable'] : false );
        if( isset($args['downloadable']) && $args['downloadable'] ) {
            $product->set_downloads(  isset($args['downloads']) ? $args['downloads'] : array() );
            $product->set_download_limit(  isset($args['download_limit']) ? $args['download_limit'] : '-1' );
            $product->set_download_expiry(  isset($args['download_expiry']) ? $args['download_expiry'] : '-1' );
        }

        if ( get_option( 'woocommerce_calc_taxes' ) === 'yes' ) {
            $product->set_tax_status(  isset($args['tax_status']) ? $args['tax_status'] : 'taxable' );
            $product->set_tax_class(  isset($args['tax_class']) ? $args['tax_class'] : '' );
        }

        if( isset($args['virtual']) && ! $args['virtual'] ) {
            $product->set_sku( isset( $args['sku'] ) ? $args['sku'] : '' );
            $product->set_manage_stock( isset( $args['manage_stock'] ) ? $args['manage_stock'] : false );
            $product->set_stock_status( isset( $args['stock_status'] ) ? $args['stock_status'] : 'instock' );
            if( isset( $args['manage_stock'] ) && $args['manage_stock'] ) {
                $product->set_stock_quantity( $args['stock_qty'] );
                $product->set_backorders( isset( $args['backorders'] ) ? $args['backorders'] : 'no' ); 
            }
        }

        $product->set_sold_individually( isset( $args['sold_individually'] ) ? $args['sold_individually'] : false );

        $product->set_weight( isset( $args['weight'] ) ? $args['weight'] : '' );
        $product->set_length( isset( $args['length'] ) ? $args['length'] : '' );
        $product->set_width( isset(  $args['width'] ) ?  $args['width']  : '' );
        $product->set_height( isset( $args['height'] ) ? $args['height'] : '' );
        if( isset( $args['shipping_class_id'] ) )
            $product->set_shipping_class_id( $args['shipping_class_id'] );

        $product->set_upsell_ids( isset( $args['upsells'] ) ? $args['upsells'] : '' );
        $product->set_cross_sell_ids( isset( $args['cross_sells'] ) ? $args['upsells'] : '' );

        $product->set_reviews_allowed( isset( $args['reviews'] ) ? $args['reviews'] : false );
        $product->set_purchase_note( isset( $args['note'] ) ? $args['note'] : '' );
        if( isset( $args['menu_order'] ) )
            $product->set_menu_order( $args['menu_order'] );

        if( !empty($args['family']) )
        {
            $category_ids = array();
            $category_id = $this->setCategory( $args['family'] );
            $category_ids[] = $category_id;
            
            if(!empty($args['family_tree']))
            {
                $parent = 0;
                foreach($args['family_tree'] as $family)
                {
                    $category_id = $this->setCategory( $args['family'], $parent );
                    $parent = $category_id;
                    $category_ids[] = $category_id;
                }
            }
        }
                
         if( isset( $category_ids ) )
            $product->set_category_ids( $category_ids );
         if( isset( $args['tag_ids'] ) )
            $product->set_tag_ids( $args['tag_ids'] );

        if( $args['image_url'] )
        {
            $uploaddir = wp_upload_dir();
            $filename = $args['sku'].'.jpg';
            $uploadfile = $uploaddir['path'] . '/' . $filename;

            $contents= file_get_contents($args['image_url']);
            $savefile = fopen($uploadfile, 'w');
            fwrite($savefile, $contents);
            fclose($savefile);

            $wp_filetype = wp_check_filetype(basename($filename), null );

            $attachment = array(
                'post_mime_type' => $wp_filetype['type'],
                'post_title' => $filename,
                'post_content' => '',
                'post_status' => 'inherit'
            );

            $attach_id = wp_insert_attachment( $attachment, $uploadfile );

            $attach_data = wp_generate_attachment_metadata( $attach_id, $uploadfile );
            wp_update_attachment_metadata( $attach_id, $attach_data );

            $product->set_image_id( isset( $attach_id ) ? $attach_id : "" );
        }
        $product_id = $product->save();
        return $product_id;
    }


    public function setCategory($name, $parent = null )
    {
        $category_id = get_term_by('name', utf8_decode($name), 'product_cat');
        if( !$category_id )
        {
            return wp_insert_term( utf8_decode($name), 'product_cat', array(
                'description' => utf8_decode($name), 
                'parent' => ( ($parent) ? $parent : '' ),
            ) );
        }else
        {
            return $category_id->term_id;
        }
    }
}
