<?php
/**
 * @package Keyinvoice
 */

namespace KeyinvoiceWordpress\Admin;
use \KeyinvoiceWordpress\API\Wp;
use \KeyinvoiceWordpress\API\Callbacks\WpCallbacks;
use SoapClient;

class Pages
{
    public $wp_api;
    public $callback;
    public $pages = array();
    public $subpages = array();
    

    public function register()
    {
        $this->wp_api = new Wp();
        $this->callback = new WpCallbacks();
        
        $this->setPages();
        $this->setSubPages();

        $this->setSettings();
        $this->setSections();
        $this->setFields();

        $this->wp_api->addPages($this->pages)->withSubPage( 'Configurações' )->addSubPages( $this->subpages )->register();
    }


    public function setPages()
    {
        $this->pages = [
            [
                'page_title' => 'Keyinvoice Configurações',
                'menu_title' => 'KEYINVOICE',
                'capability' => 'manage_options',
                'menu_slug' => 'keyinvoice',
                'callback' => [ $this->callback, 'configPage' ],
                'icon_url' => KI_PLUGIN_URL . 'assets/images/icon_keyinvoice.png',
                'position' => 110, 
            ]
        ];
    }

    public function setSubPages()
    {
        $this->subpages = [
			[
				'parent_slug' => 'keyinvoice', 
				'page_title' => 'Keyinvoice Produtos', 
				'menu_title' => 'Produtos', 
				'capability' => 'manage_options', 
				'menu_slug' => 'keyinvoice_produtos', 
				'callback' => [ $this->callback, 'productsPage' ]
            ],
			[
				'parent_slug' => 'keyinvoice', 
				'page_title' => 'Keyinvoice Encomendas', 
				'menu_title' => 'Encomendas', 
				'capability' => 'manage_options', 
				'menu_slug' => 'keyinvoice_encomendas', 
				'callback' => [ $this->callback, 'ordersPage' ]
            ],
			[
				'parent_slug' => 'keyinvoice', 
				'page_title' => 'Keyinvoice Ajuda', 
				'menu_title' => 'Ajuda', 
				'capability' => 'manage_options', 
				'menu_slug' => 'keyinvoice_ajuda', 
				'callback' => [ $this->callback, 'helpPage' ]
            ]
        ];
    }

    public function setSettings()
	{
		$args = [
			[
				'option_group' => 'keyinvoice_configs_options_group',
				'option_name' => 'url_api',
				'callback' => [ $this->callback, 'KeyConfigOptionsGroup' ]
			],
			[
				'option_group' => 'keyinvoice_configs_options_group',
				'option_name' => 'api_key',
				'callback' => [ $this->callback, 'KeyConfigOptionsGroup' ]
            ],
			[
				'option_group' => 'keyinvoice_configs_options_group',
				'option_name' => 'api_doctype',
				'callback' => [ $this->callback, 'KeyConfigOptionsGroup' ]
            ],
			[
				'option_group' => 'keyinvoice_configs_options_group',
				'option_name' => 'api_draft'
            ],
			[
				'option_group' => 'keyinvoice_configs_options_group',
				'option_name' => 'api_send_email',
            ],
			[
				'option_group' => 'keyinvoice_configs_options_group',
				'option_name' => 'api_final_consumer'
            ],
			[
				'option_group' => 'keyinvoice_configs_options_group',
				'option_name' => 'api_docs_ref'
            ]
		];

		$this->wp_api->setSettings( $args );
	}

	public function setSections()
	{
		$args = [
			[
				'id' => 'keyinvoice_configs',
				'title' => 'Settings',
				'callback' => [ $this->callback, 'KeyinvoiceConfigsSection' ],
				'page' => 'keyinvoice'
            ],
			[
				'id' => 'keyinvoice_export_orders',
				'title' => 'Exportação de Encomendas',
				'page' => 'keyinvoice'
            ],
		];

		$this->wp_api->setSections( $args );
	}

	public function setFields()
	{
		$args = [
			[
				'id' => 'url_api',
				'title' => 'URL Api',
				'callback' => [ $this->callback, 'urlAPIConfig' ],
				'page' => 'keyinvoice',
				'section' => 'keyinvoice_configs',
				'args' => [
					'label_for' => 'url_api',
					'class' => 'example-class'
                ]
            ],
			[
				'id' => 'api_key',
				'title' => 'Chave API',
				'callback' => [ $this->callback, 'keyAPIConfig' ],
				'page' => 'keyinvoice',
				'section' => 'keyinvoice_configs',
				'args' => [
					'label_for' => 'api_key',
					'class' => 'example-class',
					'text' => 'Para obter estes dados entre na sua conta KEYINVOICE e clique em Configurações -» separador “API KeyInvoice'
                ]
            ],
			[
				'id' => 'test_api',
				'title' => 'Testar Configuração',
				'callback' => [ $this->callback, 'keyBtn' ],
				'page' => 'keyinvoice',
				'section' => 'keyinvoice_configs',
				'args' => [
					'label_for' => 'api_key',
					'class' => 'example-class'
                ]
            ],
			[
				'id' => 'api_doctype',
				'title' => 'Tipo de Documento',
				'callback' => [ $this->callback, 'keySelect' ],
				'page' => 'keyinvoice',
				'section' => 'keyinvoice_export_orders',
				'args' => [
					'label_for' => 'api_doctype',
					'class' => 'example-class',
					'fields' => [ 
									'13' => 'Encomenda', 
									'4' => 'Factura', 
									'32' => 'Factura Simplificada', 
									'34' => 'Factura Recibo' 
								]
                ]
			],
			[
				'id' => 'api_draft',
				'title' => 'Manter em Rascunho?',
				'callback' => [ $this->callback, 'keyCheckbox' ],
				'page' => 'keyinvoice',
				'section' => 'keyinvoice_export_orders',
				'args' => [
					'label_for' => 'api_draft',
					'class' => 'example-class',
					'text' => 'Indique se pretende que o documento seja exportado em modo rascunho ou se que seja criado e finalizado automaticamente no seu programa de faturaçã'
				]
            ],
			[
				'id' => 'api_send_email',
				'title' => 'Enviar documento por email?',
				'callback' => [ $this->callback, 'keyCheckbox' ],
				'page' => 'keyinvoice',
				'section' => 'keyinvoice_export_orders',
				'args' => [
					'label_for' => 'api_send_email',
					'class' => 'example-class',
					'text' => 'Indique se pretende que o documento seja enviado por email para o cliente logo que seja exportado em modo finalizado. Esta opção não se aplica documentos exportados como rascunho nem a clientes que não indiquem o email no processo de encomenda. O envio é feito pelo programa de faturação pelo que terá de ter um email configurado (Configurações -» separador Servidor Email'
				]
            ],
			[
				'id' => 'api_final_consumer',
				'title' => ' Permitir facturas a consumidor final?',
				'callback' => [ $this->callback, 'keyCheckbox' ],
				'page' => 'keyinvoice',
				'section' => 'keyinvoice_export_orders',
				'args' => [
					'label_for' => 'api_final_consumer',
					'class' => 'example-class',
					'text' => 'Indique se pretende que o documento seja criado associado ao Consumidor Final caso o NIF introduzido não seja válido (não aplicável a Encomendas)'
				]
			],
			[
				'id' => 'api_docs_ref',
				'title' => 'Referência de Documento',
				'callback' => [ $this->callback, 'keyText' ],
				'page' => 'keyinvoice',
				'section' => 'keyinvoice_export_orders',
				'args' => [
					'label_for' => 'api_docs_ref',
					'class' => 'example-class',
					'placeholder' => 'Doc. Ref',
					'text' => 'Se diferente de vazio, o campo DocRef no programa de facturação será preenchido com o prefixo que aqui preencher, seguido do número da encomenda exportada.'
				]
			]
		];
		$this->wp_api->setFields( $args );
	}
}